import { Component, OnInit } from "@angular/core";
import { ToDoItem } from "../../Interfaces/ToDoItem";
import { TodoService } from "../../services/todo.service";

@Component({
  selector: "app-home",
  templateUrl: "./home.component.html",
  styleUrls: ["./home.component.css"],
})
export class HomeComponent implements OnInit {
  public todoItems: ToDoItem[] = [];
  public tempTodo: ToDoItem;

  constructor(private service: TodoService) {}

  ngOnInit() {
    this.getTodo();
  }

  editTodo(todo: ToDoItem): void {
    this.tempTodo = {
      id: todo.id,
      description: todo.description,
      isDone: todo.isDone,
    };
  }

  cancelEdit(): void {
    this.tempTodo = {
      id: 0,
      description: "",
      isDone: false,
    };
  }

  getTodo(): void {
    this.service.getAll().subscribe(
      (data) => {
        this.todoItems = data;
      },
      (error) => console.error(error)
    );
    this.cancelEdit();
  }

  saveTodo(todo: ToDoItem): void {
    if (todo.id != 0) {
      this.service.update(todo).subscribe(
        (data) => {
          const index = this.todoItems.findIndex((e) => e.id === todo.id);
          this.todoItems[index] = todo;
          this.cancelEdit();
          //this.getTodo();
        },
        (error) => console.error(error)
      );
    } else {
      todo.id = 0;
      this.service.add(todo).subscribe(
        (data: ToDoItem) => {
          this.todoItems.push(data);
          this.cancelEdit();
          //this.getTodo();
        },
        (error) => console.error(error)
      );
    }
  }

  deleteTodo(todo: ToDoItem): void {
    this.service.remove(todo).subscribe(
      (data) => {
        this.todoItems = this.todoItems.filter((t) => t !== todo);
      },
      (error) => console.error(error)
    );
  }
}
