import { Injectable, Inject } from "@angular/core";
import { HttpClient, HttpHeaders } from "@angular/common/http";
import { ToDoItem } from "../Interfaces/ToDoItem";

@Injectable({
  providedIn: "root",
})
export class TodoService {
  private headers: HttpHeaders;
  private accessPointUrl: string = "";

  constructor(private http: HttpClient, @Inject("BASE_URL") baseUrl: string) {
    this.accessPointUrl = baseUrl + "api/ToDo";
  }

  public getAll() {
    return this.http.get<ToDoItem[]>(this.accessPointUrl, {
      headers: this.headers,
    });
  }

  public get(id: number) {
    return this.http.get<ToDoItem>(this.accessPointUrl + "/" + id, {
      headers: this.headers,
    });
  }

  public add(payload: ToDoItem) {
    return this.http.post(this.accessPointUrl, payload, {
      headers: this.headers,
    });
  }

  public remove(payload: ToDoItem) {
    return this.http.delete(this.accessPointUrl + "/" + payload.id, {
      headers: this.headers,
    });
  }

  public update(payload: ToDoItem) {
    return this.http.put(this.accessPointUrl, payload, {
      headers: this.headers,
    });
  }
}
